@@ -643,8 +643,9 @@ def regression_spectra(
     step_size : int
         Step size for shifting the window.
     n_sub_windows : int
-        Should we split the window into a set of sub-windows and average the
-        spectra for each sub-window.
+        We split the window into a number of sub-windows and average the
+        spectra for each sub-window. window_length must be divisible by
+        n_sub_windows.
     return_weights : bool
         Should we return the weights for subject-specific PSDs?
         Useful for calculating the group average PSD.
@@ -686,6 +687,9 @@ def regression_spectra(
     if frequency_range is None:
         frequency_range = [0, sampling_frequency / 2]
 
+    if window_length % n_sub_windows != 0:
+        raise ValueError("window_length must be divisible by n_sub_windows.")
+
     # Do we calculate cross spectral densities?
     calc_cpsd = not psd_only
 
@@ -703,9 +707,8 @@ def regression_spectra(
     # Remove the data points lost due to separating into sequences
     data = [d[: a.shape[0]] for d, a in zip(data, alpha)]
 
-    # Calculate a time-varying PSD
-    Pt = []
-    at = []
+    # Calculate a time-varying PSD and regress to get the mode PSDs
+    Pj = []
     for i in range(n_subjects):
         if n_subjects > 1:
             print(f"Subject {i}:")
@@ -724,15 +727,7 @@ def regression_spectra(
             step_size=step_size,
             n_sub_windows=n_sub_windows,
         )
-        Pt.append(p)
-        at.append(a)
-
-    # Regress the time-varying PSD with alpha to get the mode PSDs
-    Pj = []
-    for i in trange(n_subjects, desc="Fitting linear regression", ncols=98):
-        coefs, intercept = regression.linear(
-            at[i], Pt[i], fit_intercept=True, normalize=True
-        )
+        coefs, intercept = regression.linear(a, p, fit_intercept=True, normalize=True)
         Pj.append([coefs, [intercept] * coefs.shape[0]])
     Pj = np.array(Pj)
 
