@@ -115,23 +115,23 @@ class VariationalInferenceModelBase(ModelBase):
 
         return super().fit(*args, **kwargs)
 
-    def initialize(
+    def multistart_initialization(
         self,
-        training_dataset,
-        epochs,
+        training_data,
+        n_epochs,
         n_init,
         **kwargs,
     ):
-        """Multi-start training.
+        """Multi-start initialization.
 
         The model is trained for a few epochs with different random initializations
         for weights and the model with the best free energy is kept.
 
         Parameters
         ----------
-        training_dataset : tensorflow.data.Dataset
+        training_data : tensorflow.data.Dataset or osl_dynamics.data.Data
             Dataset to use for training.
-        epochs : int
+        n_epochs : int
             Number of epochs to train the model.
         n_init : int
             Number of initializations.
@@ -152,11 +152,10 @@ class VariationalInferenceModelBase(ModelBase):
         best_free_energy = np.Inf
         for n in range(n_init):
             print(f"Initialization {n}")
-            self.reset_weights()
-            self.compile()
+            self.reset()
             history = self.fit(
-                training_dataset,
-                epochs=epochs,
+                training_data,
+                epochs=n_epochs,
                 **kwargs,
             )
             free_energy = history.history["loss"][-1]
@@ -167,10 +166,8 @@ class VariationalInferenceModelBase(ModelBase):
                 best_history = history
 
         print(f"Using initialization {best_initialization}")
-        self.model.set_weights(best_weights)
-        if self.config.do_kl_annealing:
-            self.reset_kl_annealing_factor()
-        self.compile()
+        self.reset()
+        self.set_weights(best_weights)
 
         return best_history
 
@@ -210,13 +207,13 @@ class VariationalInferenceModelBase(ModelBase):
 
         return predictions_dict
 
-    def get_alpha(self, inputs, *args, concatenate=False, **kwargs):
+    def get_alpha(self, inputs, concatenate=False):
         """Mode mixing factors, alpha.
 
         Parameters
         ----------
-        inputs : tensorflow.data.Dataset
-            Prediction dataset.
+        inputs : tensorflow.data.Dataset or osl_dynamics.data.Data
+            Prediction data.
         concatenate : bool
             Should we concatenate alpha for each subject?
 
@@ -231,10 +228,12 @@ class VariationalInferenceModelBase(ModelBase):
                 inputs, *args, concatenate=concatenate, **kwargs
             )
 
-        inputs = self._make_dataset(inputs)
+        inputs = self.make_dataset(inputs, concatenate=concatenate)
+
+        print("Getting alpha:")
         outputs = []
         for dataset in inputs:
-            alpha = self.predict(dataset, *args, **kwargs)["alpha"]
+            alpha = self.predict(dataset)["alpha"]
             alpha = np.concatenate(alpha)
             outputs.append(alpha)
 
@@ -243,15 +242,15 @@ class VariationalInferenceModelBase(ModelBase):
 
         return outputs
 
-    def get_mode_time_courses(self, inputs, *args, concatenate=False, **kwargs):
+    def get_mode_time_courses(self, inputs, concatenate=False):
         """Get mode time courses.
 
         This method is used to get mode time courses for the multi-time-scale model.
 
         Parameters
         ----------
-        inputs : tensorflow.data.Dataset
-            Prediction dataset.
+        inputs : tensorflow.data.Dataset or osl_dynamics.data.Data
+            Prediction data.
         concatenate : bool
             Should we concatenate alpha for each subject?
 
@@ -267,12 +266,13 @@ class VariationalInferenceModelBase(ModelBase):
         if not self.config.multiple_dynamics:
             raise ValueError("Please use get_alpha for a single time scale model.")
 
-        inputs = self._make_dataset(inputs)
+        inputs = self.make_dataset(inputs, concatenate=concatenate)
 
+        print("Getting mode time courses:")
         outputs_alpha = []
         outputs_gamma = []
         for dataset in inputs:
-            predictions = self.predict(dataset, *args, **kwargs)
+            predictions = self.predict(dataset)
 
             alpha = predictions["alpha"]
             gamma = predictions["gamma"]
@@ -294,7 +294,7 @@ class VariationalInferenceModelBase(ModelBase):
 
         Parameters
         ----------
-        dataset : tensorflow.data.Dataset
+        dataset : tensorflow.data.Dataset or osl_dynamics.data.Data
             Dataset to calculate losses for.
 
         Returns
@@ -304,24 +304,25 @@ class VariationalInferenceModelBase(ModelBase):
         kl_loss : float
             KL divergence loss.
         """
-        if isinstance(dataset, list):
-            predictions = [self.predict(subject) for subject in dataset]
-            ll_loss = np.mean([np.mean(p["ll_loss"]) for p in predictions])
-            kl_loss = np.mean([np.mean(p["kl_loss"]) for p in predictions])
-
-        else:
-            predictions = self.predict(dataset)
-            ll_loss = np.mean(predictions["ll_loss"])
-            kl_loss = np.mean(predictions["kl_loss"])
-
+        dataset = self.make_dataset(dataset, concatenate=True)
+        print("Getting losses:")
+        predictions = self.predict(dataset)
+        ll_loss = np.mean(predictions["ll_loss"])
+        kl_loss = np.mean(predictions["kl_loss"])
         return ll_loss, kl_loss
 
     def free_energy(self, dataset):
         """Calculates the variational free energy of a dataset.
 
+        Note, this method returns a free energy which may have a significantly
+        smaller KL loss. This is because during training we sample from the
+        posterior, however, when we're evaluating the model, we take the maximum
+        a posteriori estimate (posterior mean). This has the effect of giving a
+        lower KL loss for a given dataset.
+
         Parameters
         ----------
-        dataset : tensorflow.data.Dataset
+        dataset : tensorflow.data.Dataset or osl_dynamics.data.Data.
             Dataset to calculate the variational free energy for.
 
         Returns
@@ -329,6 +330,7 @@ class VariationalInferenceModelBase(ModelBase):
         free_energy : float
             Variational free energy for the dataset.
         """
+        dataset = self.make_dataset(dataset, concatenate=True)
         ll_loss, kl_loss = self.losses(dataset)
         free_energy = ll_loss + kl_loss
         return free_energy
